94057c3d3e
PR Test (NPU) / check-changes (push) Has been cancelled
PR Test (NPU) / pr-gate (push) Has been cancelled
PR Test (NPU) / set-image-config (push) Has been cancelled
PR Test (NPU) / stage-b-test-1-npu-a2 (0) (push) Has been cancelled
PR Test (NPU) / stage-b-test-1-npu-a2 (1) (push) Has been cancelled
PR Test (NPU) / stage-b-test-2-npu-a2 (0) (push) Has been cancelled
PR Test (NPU) / stage-b-test-2-npu-a2 (1) (push) Has been cancelled
PR Test (NPU) / stage-b-test-4-npu-a3 (push) Has been cancelled
PR Test (NPU) / stage-b-test-16-npu-a3 (push) Has been cancelled
PR Test (NPU) / multimodal-gen-test-1-npu-a3 (push) Has been cancelled
PR Test (NPU) / multimodal-gen-test-2-npu-a3 (push) Has been cancelled
PR Test (Arm64) / pr-gate (push) Has been cancelled
PR Test (Arm64) / check-changes (push) Has been cancelled
PR Test (Arm64) / build-test (push) Has been cancelled
PR Test (sgl-router) / gate (push) Has been cancelled
PR Test (sgl-router) / tier-1 — lint (push) Has been cancelled
PR Test (sgl-router) / tier-2 — build + test (push) Has been cancelled
PR Test (sgl-router) / tier-3 — docker (placeholder) (push) Has been cancelled
PR Test (sgl-router) / tier-3 — k8s integration (push) Has been cancelled
PR Test (sgl-router) / tier-3 — e2e (push) Has been cancelled
PR Test (sgl-router) / finish (push) Has been cancelled
PR Test (NPU) / single-node-poc (map[name:qwen3_6_27b_w8a8_1p_in64k_out1k_50ms runner:linux-aarch64-a3-2 test_case:test/registered/ascend/performance/qwen3_6_27b/test_npu_qwen3_6_27b_w8a8_1p_in64k_out1k_50ms.py test_type:perf]) (push) Has been cancelled
PR Test (NPU) / pr-test-npu-finish (push) Has been cancelled
PR Test (Xeon) / pr-gate (push) Has been cancelled
PR Test (Xeon) / check-changes (push) Has been cancelled
PR Test (Xeon) / build-test (, xeon-gnr, base-b-test-cpu) (push) Has been cancelled
PR Test (XPU) / check-changes (push) Has been cancelled
PR Test (XPU) / pr-gate (push) Has been cancelled
PR Test (XPU) / stage-a-test-1-gpu-xpu (push) Has been cancelled
PR Test (XPU) / wait-for-stage-a (push) Has been cancelled
PR Test (XPU) / stage-b-test-1-gpu-xpu (push) Has been cancelled
PR Test (XPU) / finish (push) Has been cancelled
CI Model Inventory / build-inventory (push) Has been cancelled
Lint / lint (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark Compilation Check (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark - Manual Policy (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark - Request Processing (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark Summary (push) Has been cancelled
PR Test (SMG) / build-wheel (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on windows (x86_64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on macos (x86_64 - auto) (push) Has been cancelled
PR Test (SMG) / python-unit-tests (push) Has been cancelled
PR Test (SMG) / unit-tests (push) Has been cancelled
PR Test (SMG) / benchmarks (push) Has been cancelled
PR Test (SMG) / chat-completions (push) Has been cancelled
PR Test (SMG) / chat-completions-4gpu (push) Has been cancelled
PR Test (SMG) / e2e (push) Has been cancelled
PR Test (SMG) / docker-build-test (push) Has been cancelled
PR Test (SMG) / k8s-integration (push) Has been cancelled
PR Test (SMG) / finish (push) Has been cancelled
PR Test (SMG) / summarize-benchmarks (push) Has been cancelled
Release SGLang Model Gateway Docker Image / publish (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on macos (aarch64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (aarch64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (x86_64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (aarch64 - musllinux_1_1) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (x86_64 - musllinux_1_1) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / Build SDist (push) Has been cancelled
Release SGLang Model Gateway to PyPI / Upload to PyPI (push) Has been cancelled
Release SGLang Kernels / build-cu129-matrix (aarch64, 12.9, 3.10, arm-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / build-cu129-matrix (x86_64, 12.9, 3.10, x64-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / release-cu129 (push) Has been cancelled
Release SGLang Kernels / build-cu130-matrix (aarch64, 13.0, 3.10, arm-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / build-cu130-matrix (x86_64, 13.0, 3.10, x64-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / release-cu130 (push) Has been cancelled
Release SGLang Kernels / build-rocm-matrix (3.10, 700) (push) Has been cancelled
Release SGLang Kernels / build-rocm-matrix (3.10, 720) (push) Has been cancelled
Release SGLang Kernels / release-rocm700 (push) Has been cancelled
Release SGLang Kernels / release-rocm720 (push) Has been cancelled
Release SGLang Kernels / build-musa43 (43, 3.10) (push) Has been cancelled
Release SGLang Kernels / release-musa43 (push) Has been cancelled
185 lines
5.4 KiB
Python
185 lines
5.4 KiB
Python
# Adapted from NVlabs/Sana sol-engine LTX2 Ada-value fusion.
|
|
#
|
|
# SPDX-License-Identifier: Apache-2.0
|
|
|
|
import torch
|
|
import triton
|
|
import triton.language as tl
|
|
|
|
|
|
@triton.jit
|
|
def _ltx2_ada_values9_kernel(
|
|
temb_ptr,
|
|
table_ptr,
|
|
out0_ptr,
|
|
out1_ptr,
|
|
out2_ptr,
|
|
out3_ptr,
|
|
out4_ptr,
|
|
out5_ptr,
|
|
out6_ptr,
|
|
out7_ptr,
|
|
out8_ptr,
|
|
rows: tl.constexpr,
|
|
hidden: tl.constexpr,
|
|
total_params: tl.constexpr,
|
|
table_stride_p: tl.constexpr,
|
|
table_stride_d: tl.constexpr,
|
|
BLOCK_N: tl.constexpr,
|
|
):
|
|
row = tl.program_id(0).to(tl.int64)
|
|
cols = tl.arange(0, BLOCK_N)
|
|
mask = cols < hidden
|
|
temb_row = temb_ptr + row * total_params * hidden
|
|
base = row * hidden + cols
|
|
|
|
table0 = tl.load(
|
|
table_ptr + 0 * table_stride_p + cols * table_stride_d,
|
|
mask=mask,
|
|
other=0.0,
|
|
).to(tl.bfloat16)
|
|
temb0 = tl.load(
|
|
temb_row + (0 * hidden + cols),
|
|
mask=mask,
|
|
other=0.0,
|
|
).to(tl.bfloat16)
|
|
table1 = tl.load(
|
|
table_ptr + 1 * table_stride_p + cols * table_stride_d,
|
|
mask=mask,
|
|
other=0.0,
|
|
).to(tl.bfloat16)
|
|
temb1 = tl.load(
|
|
temb_row + (1 * hidden + cols),
|
|
mask=mask,
|
|
other=0.0,
|
|
).to(tl.bfloat16)
|
|
table2 = tl.load(
|
|
table_ptr + 2 * table_stride_p + cols * table_stride_d,
|
|
mask=mask,
|
|
other=0.0,
|
|
).to(tl.bfloat16)
|
|
temb2 = tl.load(
|
|
temb_row + (2 * hidden + cols),
|
|
mask=mask,
|
|
other=0.0,
|
|
).to(tl.bfloat16)
|
|
table3 = tl.load(
|
|
table_ptr + 3 * table_stride_p + cols * table_stride_d,
|
|
mask=mask,
|
|
other=0.0,
|
|
).to(tl.bfloat16)
|
|
temb3 = tl.load(
|
|
temb_row + (3 * hidden + cols),
|
|
mask=mask,
|
|
other=0.0,
|
|
).to(tl.bfloat16)
|
|
table4 = tl.load(
|
|
table_ptr + 4 * table_stride_p + cols * table_stride_d,
|
|
mask=mask,
|
|
other=0.0,
|
|
).to(tl.bfloat16)
|
|
temb4 = tl.load(
|
|
temb_row + (4 * hidden + cols),
|
|
mask=mask,
|
|
other=0.0,
|
|
).to(tl.bfloat16)
|
|
table5 = tl.load(
|
|
table_ptr + 5 * table_stride_p + cols * table_stride_d,
|
|
mask=mask,
|
|
other=0.0,
|
|
).to(tl.bfloat16)
|
|
temb5 = tl.load(
|
|
temb_row + (5 * hidden + cols),
|
|
mask=mask,
|
|
other=0.0,
|
|
).to(tl.bfloat16)
|
|
table6 = tl.load(
|
|
table_ptr + 6 * table_stride_p + cols * table_stride_d,
|
|
mask=mask,
|
|
other=0.0,
|
|
).to(tl.bfloat16)
|
|
temb6 = tl.load(
|
|
temb_row + (6 * hidden + cols),
|
|
mask=mask,
|
|
other=0.0,
|
|
).to(tl.bfloat16)
|
|
table7 = tl.load(
|
|
table_ptr + 7 * table_stride_p + cols * table_stride_d,
|
|
mask=mask,
|
|
other=0.0,
|
|
).to(tl.bfloat16)
|
|
temb7 = tl.load(
|
|
temb_row + (7 * hidden + cols),
|
|
mask=mask,
|
|
other=0.0,
|
|
).to(tl.bfloat16)
|
|
table8 = tl.load(
|
|
table_ptr + 8 * table_stride_p + cols * table_stride_d,
|
|
mask=mask,
|
|
other=0.0,
|
|
).to(tl.bfloat16)
|
|
temb8 = tl.load(
|
|
temb_row + (8 * hidden + cols),
|
|
mask=mask,
|
|
other=0.0,
|
|
).to(tl.bfloat16)
|
|
|
|
tl.store(out0_ptr + base, (table0 + temb0).to(tl.bfloat16), mask=mask)
|
|
tl.store(out1_ptr + base, (table1 + temb1).to(tl.bfloat16), mask=mask)
|
|
tl.store(out2_ptr + base, (table2 + temb2).to(tl.bfloat16), mask=mask)
|
|
tl.store(out3_ptr + base, (table3 + temb3).to(tl.bfloat16), mask=mask)
|
|
tl.store(out4_ptr + base, (table4 + temb4).to(tl.bfloat16), mask=mask)
|
|
tl.store(out5_ptr + base, (table5 + temb5).to(tl.bfloat16), mask=mask)
|
|
tl.store(out6_ptr + base, (table6 + temb6).to(tl.bfloat16), mask=mask)
|
|
tl.store(out7_ptr + base, (table7 + temb7).to(tl.bfloat16), mask=mask)
|
|
tl.store(out8_ptr + base, (table8 + temb8).to(tl.bfloat16), mask=mask)
|
|
|
|
|
|
def ltx2_ada_values9(
|
|
scale_shift_table: torch.Tensor,
|
|
timestep: torch.Tensor,
|
|
) -> tuple[torch.Tensor, ...]:
|
|
if timestep.ndim != 3:
|
|
raise ValueError("timestep must have shape [B, S, 9 * D]")
|
|
if not timestep.is_cuda or timestep.dtype != torch.bfloat16:
|
|
raise ValueError("timestep must be a CUDA bfloat16 tensor")
|
|
if not timestep.is_contiguous():
|
|
raise ValueError("timestep must be contiguous")
|
|
if scale_shift_table.ndim != 2 or scale_shift_table.shape[0] != 9:
|
|
raise ValueError("scale_shift_table must have shape [9, D]")
|
|
if (
|
|
not scale_shift_table.is_cuda
|
|
or scale_shift_table.dtype not in (torch.bfloat16, torch.float32)
|
|
or scale_shift_table.stride(-1) != 1
|
|
):
|
|
raise ValueError(
|
|
"scale_shift_table must be CUDA, bf16/fp32, last-dim contiguous"
|
|
)
|
|
|
|
total_params = int(scale_shift_table.shape[0])
|
|
hidden = int(scale_shift_table.shape[1])
|
|
if hidden <= 0 or timestep.shape[-1] != total_params * hidden:
|
|
raise ValueError("timestep last dim must equal 9 * hidden")
|
|
if hidden % 256 != 0 or hidden > 8192:
|
|
raise ValueError("hidden size is outside the supported LTX2 fast-path range")
|
|
|
|
batch, seq, _ = timestep.shape
|
|
rows = int(batch * seq)
|
|
outs = tuple(
|
|
torch.empty((batch, seq, hidden), device=timestep.device, dtype=timestep.dtype)
|
|
for _ in range(9)
|
|
)
|
|
_ltx2_ada_values9_kernel[(rows,)](
|
|
timestep,
|
|
scale_shift_table,
|
|
*outs,
|
|
rows,
|
|
hidden,
|
|
total_params,
|
|
scale_shift_table.stride(0),
|
|
scale_shift_table.stride(1),
|
|
BLOCK_N=triton.next_power_of_2(hidden),
|
|
num_warps=4 if hidden >= 4096 else 8,
|
|
)
|
|
return outs
|